# Puppet resource definition for wrapper on updates on a server - firmware, software drivers, and HA config
#
# $path = path to the firmware binary OR the catalog.xml file for the idrac
# $force_restart = automatically restart the iDRAC server if reboots are needed (don't wait on user to reboot)
# $asm_hostname = the IP address of the appliance to get the firmware binaries from
# $server_firmware = List of firmware to install on server
# $server_software = List of software packages to install on the OS on server
define asm::server_update (
  $install_type    = "uri",
  $force_restart   = false,
  $path            = undef,
  $asm_hostname    = undef,
  $esx_hostname    = undef,
  $esx_password    = undef,
  $server_firmware = undef,
  $server_software = undef,
  $vcenter_cert    = undef,
  $vcenter_ha_config = undef
) {
  # Validation checks
  if (!$server_firmware and !$server_software) {
    fail("Specify atleast server_firmware or server_software parameter(s)")
  }

  # Create desired share and ensure NFS service is running
  if $path =~ /^(.+)\/([^\/]+)$/ {
    $share_path = $1
  }
  file_line { "create_share":
    path   => '/etc/exports',
    line   => "$share_path     *(ro,sync,no_subtree_check)",
    notify => Service["nfs"],
  }
  service { "nfs":
    ensure  => running,
    require => File_line['create_share']
  }

  if ($esx_hostname != undef) and ($force_restart == true) {
    #  If esx_hostname is defined, invoke the ESX update resource
    asm::server_update_esx { "server_update_esx" :
      install_type       => $install_type,
      force_restart      => $force_restart,
      path               => $path,
      asm_hostname       => $asm_hostname,
      esx_hostname       => $esx_hostname,
      esx_password       => $esx_password,
      server_firmware    => $server_firmware,
      server_software    => $server_software,
      vcenter_cert       => $vcenter_cert,
      vcenter_ha_config  => $vcenter_ha_config,
      idrac_name         => $name,
      require            => File_line['create_share']
    }
  }
  elsif ($server_firmware != undef)  {
    #  If purely a server firmware update is needed, invoke firmware only update
    asm::server_update_firmware { "server_update_firmware" :
      install_type       => $install_type,
      force_restart      => $force_restart,
      path               => $path,
      asm_hostname       => $asm_hostname,
      server_firmware    => $server_firmware,
      idrac_name         => $name,
      require            => File_line['create_share']
    }
  }
}
