# Puppet resource definition for wrapper on updates on ESX - firmware, software drivers, and HA config
#
# $path = path to the firmware binary OR the catalog.xml file for the idrac
# $force_restart = automatically restart the iDRAC server if reboots are needed (don't wait on user to reboot)
# $asm_hostname = the IP address of the appliance to get the firmware binaries from
# $server_firmware = List of firmware to install on server
# $server_software = List of software packages to install on the OS on server
define asm::server_update_esx (
  $install_type       = "uri",
  $force_restart      = false,
  $path               = undef,
  $asm_hostname       = undef,
  $esx_hostname       = undef,
  $esx_password       = undef,
  $server_firmware    = undef,
  $server_software    = undef,
  $vcenter_cert       = undef,
  $vcenter_ha_config  = undef,
  $idrac_name         = undef
) {
  # Validation checks
  if ($install_type != 'uri' and $server_software != undef) {
    fail("server_software parameter is only supported for uri install_type")
  }

  # Define iDRAC transport connection
  transport {'idrac':
    provider => 'device_file',
    name     => $idrac_name
  }

  # Define transport connections to vCenter/ESX and HA configs
  if ($vcenter_cert != undef) {

    transport { 'firstVcenter' :
      options  => {'insecure' => true, 'endpoint' => $vcenter_cert},
      provider => 'device_file'
    }
    transport { 'secondVcenter' :
      options  => {'insecure' => true, 'endpoint' => $vcenter_cert},
      provider => 'device_file'
    }

    if ($vcenter_ha_config != undef) {
      $vcenter_options = parsejson($vcenter_ha_config)
      asm::cluster::vcenter_hadrs { $vcenter_cert :
        cluster_path    => $vcenter_options['cluster_path'],
        vcenter_options =>  {'insecure' => true},
        ha_config       => 'true',
        drs_config      => $vcenter_options['drs_config'],
        require         => Esx_connection_wait['wait_for_esxi_services'],
        transport       => Transport['secondVcenter']
      }
      esx_reconfigureha { "Reconfig#{$esx_hostname}" :
        host      => $esx_hostname,
        ensure    => present,
        force     => true,
        transport => Transport['secondVcenter'],
        require   => [Asm::Cluster::Vcenter_hadrs[$vcenter_cert],Esx_connection_wait['wait_for_esxi_services'],Esx_maintmode['disable']]
      }
    }
  }
  else {

    transport { 'firstVcenter' :
      username => 'root',
      password => asm_cipher_decrypt($esx_password),
      server   => $esx_hostname,
      options  => { 'insecure' => true },
    }
    transport { 'secondVcenter' :
      username => 'root',
      password => asm_cipher_decrypt($esx_password),
      server   => $esx_hostname,
      options  => { 'insecure' => true },
    }
  }

  # Define ESX maintenance modes and wait_for_esxi_services resources
  esx_maintmode { 'enable' :
    ensure                   => present,
    timeout                  => 450,
    transport                => Transport['firstVcenter'],
    evacuate_powered_off_vms => true,
    host                     => $esx_hostname,
    vsan_action              => "noAction"
  }

  esx_maintmode { 'disable':
    ensure                   => absent,
    timeout                  => 300,
    transport                => Transport['secondVcenter'],
    host                     => $esx_hostname,
    evacuate_powered_off_vms => true,
  }
  if ($server_firmware != undef ) {
    esx_connection_wait { 'wait_for_esxi_services' :
      ensure      => present,
      init_sleep  => 300,
      max_wait    => 900,
      host        => $esx_hostname,
      transport   => Transport['firstVcenter'],
    }
  }
  else {
    # If there is no server firmware, we hardly need to wait for ESX services
    # NOTE: We do need this resource for few seconds to trigger DRS / HA
    esx_connection_wait { 'wait_for_esxi_services' :
      ensure      => present,
      init_sleep  => 5,
      max_wait    => 60,
      host        => $esx_hostname,
      transport   => Transport['firstVcenter'],
    }
  }

  # Using resources we just defined, we put the host in maintenance mode before
  # update, do update, and take out of maintenance mode on successful update
  if $install_type != 'uri' {
    # For none-URI method, simply do firmware update with enable/disable ESX maintenance
    idrac_fw_update { "update":
      name          => $idrac_name,
      asm_hostname  => $asm_hostname,
      path          => $path,
      force_restart => $force_restart,
      require       => Esx_maintmode['enable'],
      before        => Esx_maintmode['disable'],
      transport     => Transport['idrac']
    }
    Idrac_fw_update['update'] -> Esx_connection_wait['wait_for_esxi_services'] -> Esx_maintmode['disable']
  }
  else {
    # For URI method, we can do software and/or firmware updates
    # with enable/disable ESX maintenance
    if ($server_firmware != undef and $server_software != undef) {

      # Define Firmware update resource
      idrac_fw_installfromuri { "idrac" :
        ensure         => present,
        force_restart  => $force_restart,
        idrac_firmware => path_to_uri($server_firmware, $asm_hostname),
        require        => Esx_maintmode['enable'],
        before         => Esx_maintmode['disable'],
        transport      => Transport['idrac']
      }

      # Define Software update resource
      esx_software_update { "esx_update" :
        ensure          => present,
        reboot_timeout  => 900,
        vibs            => parse_for_vib_path($server_software),
        nfs_hostname    => $asm_hostname,
        host            => $esx_hostname,
        require         => Esx_maintmode['enable'],
        before          => Esx_maintmode['disable'],
        transport       => Transport['firstVcenter']
      }

      # Set ordering for update with maintenance
      Esx_maintmode['enable'] -> Idrac_fw_installfromuri['idrac'] -> Esx_connection_wait['wait_for_esxi_services'] -> Esx_software_update['esx_update'] -> Esx_maintmode['disable']
    }
    elsif ($server_firmware != undef) { # i.e. Firmware only update

      # Define Firmware update resource
      idrac_fw_installfromuri { "idrac" :
        ensure         => present,
        force_restart  => $force_restart,
        idrac_firmware => path_to_uri($server_firmware, $asm_hostname),
        require        => Esx_maintmode['enable'],
        before         => Esx_maintmode['disable'],
        transport      => Transport['idrac']
      }

      # Set ordering for update with maintenance
      Esx_maintmode['enable'] -> Idrac_fw_installfromuri['idrac'] -> Esx_connection_wait['wait_for_esxi_services'] -> Esx_maintmode['disable']
    }
    else { # i.e Software only update

      # Define Software update resource
      esx_software_update { "esx_update" :
        ensure          => present,
        reboot_timeout  => 900,
        vibs            => parse_for_vib_path($server_software),
        nfs_hostname    => $asm_hostname,
        host            => $esx_hostname,
        require         => Esx_maintmode['enable'],
        before          => Esx_maintmode['disable'],
        transport       => Transport['firstVcenter']
      }

      # Set ordering for update with maintenance
      Esx_maintmode['enable'] ->  Esx_connection_wait['wait_for_esxi_services'] -> Esx_software_update['esx_update'] -> Esx_maintmode['disable']
    }
  }
}
